/*
  StrobeVideo - Analyse

  Erstellt ein Stroboskopbild aus einer Videodatei und lässt stellt Werkzeuge zur Verfügung, um dieses Bild im Sinne einer Videoanalyse auszuwerten.

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


let input;
let video;

let img;
let img_reference;
let img_big;
let img_canvas;

let scale;
let scale_value;
let scale_length;

let fps;
const fps_max = 12;
const fps_default = 24;
let threshold = 60;

let render_reference;
let render;
let new_video;

let framerates = [];
let average_framerates;
let framerate_error;




function handleFile(file) {
  if (file.type === 'video') {
    video = createVideo(file.data);
    video.hide();
    new_video = true;
  } else {
    video = null;
  }
}




function pauseRender() {
  video.pause();

  if (render == 1) {
    render = 0;
    render_reference = false;
    new_video = false;

    let sum_framerates = 0;
    if (framerates.length > 3) {
      for (let i = 3; i < framerates.length; i++)
        sum_framerates += framerates[i];

      average_framerates = sum_framerates / (framerates.length - 3);

      if (average_framerates > (fps + 0.6) || average_framerates < (fps - 0.6))
        framerate_error = true;
    } else {
      average_framerates = fps;
      framerate_error = false;
    }

    frameRate(fps_default);

  } else {
    if (render == 2) {
      render = 1;
      render_reference = true;
      new_video = false;
    }
  }
}



function renderImage() {
  if (video) {
    video.play().time(0);
    render_reference = true;
    if (new_video)
      render = 2;
    else
      render = 1;
  }
}



function renderReference() {
  if (video) {
    framerates.splice(0, framerates.length);

    video.loadPixels();

    img = createImage(video.width, video.height);
    img_reference = createImage(video.width, video.height);

    img_reference.loadPixels();
    img.loadPixels();

    video.pause().time(0);
    video.updatePixels();
    video.loadPixels();

    for (let y = 0; y < video.height; y++) {
      for (let x = 0; x < video.width; x++) {
        let index = (x + (y * video.width)) * 4;

        img_reference.pixels[index] = video.pixels[index];
        img_reference.pixels[index + 1] = video.pixels[index + 1];
        img_reference.pixels[index + 2] = video.pixels[index + 2];
        img_reference.pixels[index + 3] = video.pixels[index + 3];

        img.pixels[index] = img_reference.pixels[index];
        img.pixels[index + 1] = img_reference.pixels[index + 1];
        img.pixels[index + 2] = img_reference.pixels[index + 2];
        img.pixels[index + 3] = img_reference.pixels[index + 3];
      }
    }
    img_reference.updatePixels();
    img.updatePixels();

    frameRate(fps);
    framerate_error = false;
    render_reference = false;
    video.play().time(0);
  }
}



function analyseFrame() {
  img.loadPixels();
  video.loadPixels();
  img_reference.loadPixels();

  for (let y = 0; y < video.height; y++) {
    for (let x = 0; x < video.width; x++) {
      let index = (x + (y * video.width)) * 4;

      let r1 = video.pixels[index];
      let g1 = video.pixels[index + 1]
      let b1 = video.pixels[index + 2]

      let r2 = img_reference.pixels[index];
      let g2 = img_reference.pixels[index + 1];
      let b2 = img_reference.pixels[index + 2];

      let dist = distSq(r1, g1, b1, r2, g2, b2);

      if (dist > threshold * threshold) {
        img.pixels[index] = r1;
        img.pixels[index + 1] = g1;
        img.pixels[index + 2] = b1;
        img.pixels[index + 3] = 255;
      }
    }
  }
  img.updatePixels();
}



function distSq(x1, y1, z1, x2, y2, z2) {
  return ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1) + (z2 - z1) * (z2 - z1));
}