/*
  StrobeVideo - Interface

  Erstellt ein Stroboskopbild aus einer Videodatei und lässt stellt Werkzeuge zur Verfügung, um dieses Bild im Sinne einer Videoanalyse auszuwerten.

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


const margin_relation = 0.007;
const textsize = 18;

let margin;
let button_width;
let button_height;

let slider;

let button_render;
let button_save;
let button_display_mode;
let button_up_down;

let interface_up;

let img_video_width;
let img_video_height;

let always_show_magnifier;
let show_magnifier;
const magnification = 0.6;

let display_mode;
let vectors = [];
let vector_colors = [];

let magn_width;
let magn_height;



function renderInstructions() {
  push()
  fill(240);
  stroke(0);
  strokeWeight(2);
  rect((windowWidth - img_video_width - 2 * margin) * 0.5 + margin, margin, img_video_width, img_video_height);

  fill(0);
  noStroke();
  textAlign(CENTER);
  textSize(textsize * margin * 0.2);
  strokeWeight(1);
  stroke(0);
  textStyle(BOLD);
  text("Mit diesem Programm kannst du Stroboskopbilder erstellen.", windowWidth * 0.5,
    img_video_height * 0.5 - 5 * textsize * 0.3 * margin - 8 * margin);
  noStroke();
  text("Wähle dazu mit 'Datei auswählen' eine Videodatei aus.", windowWidth * 0.5,
    img_video_height * 0.5 - 4 * textsize * 0.3 * margin - 5 * margin);
  textStyle(NORMAL);
  text("Überspringe diesen Schritt um das Beispielvideo zu verwenden.", windowWidth * 0.5,
    img_video_height * 0.5 - 3 * textsize * 0.3 * margin - 5 * margin);
  text("Es werden Videos im Querformat als mp4- oder MOV-Datei unterstützt.", windowWidth * 0.5,
    img_video_height * 0.5 - 2 * textsize * 0.3 * margin - 5 * margin);
  textStyle(BOLD);
  text("Mit dem Schieberegler kannst du die gewünschte Bildfrequenz einstellen.", windowWidth * 0.5,
    img_video_height * 0.5 - 1 * textsize * 0.3 * margin - 3 * margin);
  textStyle(NORMAL);
  text("fps steht für 'frames per seconds', also 'Bilder pro Sekunde'.", windowWidth * 0.5,
    img_video_height * 0.5 - 3 * margin);
  textStyle(BOLD);
  text("Mit 'Rendern' wird das Stroboskopbild erzeugt.", windowWidth * 0.5,
    img_video_height * 0.5 + 1 * textsize * 0.3 * margin - 1 * margin);
  textStyle(NORMAL);
  text("Dazu wird das Video ein- bis zweimal abgespielt. Warte diese Zeit ab.", windowWidth * 0.5,
    img_video_height * 0.5 + 2 * textsize * 0.3 * margin - 1 * margin);
  text("Sollte das Bild auf dem Kopf stehen, nimm das Video erneut um 180° gedreht auf.", windowWidth * 0.5,
    img_video_height * 0.5 + 3 * textsize * 0.3 * margin - 1 * margin);
  textStyle(BOLD);
  text("Mit 'Ansicht' wechselst du zwischen der Längenmessung und Vektoransichten.", windowWidth * 0.5,
    img_video_height * 0.5 + 4 * textsize * 0.3 * margin + 1 * margin);
  if (display_mode == 0) {
    text("Miss mit der schwarzen Strecke die Länge des Vergleichsmaßes aus.", windowWidth * 0.5,
      img_video_height * 0.5 + 5 * textsize * 0.3 * margin + 3 * margin);
    textStyle(NORMAL);
    text("Trage dort die reale Länge ein. Zoome mit einem Doppelklick wieder heraus.", windowWidth * 0.5,
      img_video_height * 0.5 + 6 * textsize * 0.3 * margin + 3 * margin);
  } else {
    text("Du kannst die Startpunkte, Endpunkte und Mitten der Pfeile bewegen.", windowWidth * 0.5,
      img_video_height * 0.5 + 5 * textsize * 0.3 * margin + 3 * margin);
    textStyle(NORMAL);
    text("Drücke die Leertaste, um den Lupenmodus zu wechseln.", windowWidth * 0.5,
      img_video_height * 0.5 + 6 * textsize * 0.3 * margin + 3 * margin);
  }
  pop();
}



function renderError() {
  push();
  stroke(255);
  strokeWeight(2);
  fill(170, 0, 0);
  textAlign(CENTER);
  textSize(textsize * margin * 0.3);
  text("FEHLER", windowWidth * 0.5,
    img_video_height * 0.5 - 2 * (textsize * 0.3 * margin + 2 * margin));
  text("Die tatsächliche Bildrate weicht im Mittel um", windowWidth * 0.5,
    img_video_height * 0.5 - textsize * 0.3 * margin - 2 * margin);
  text(str((fps - average_framerates).toFixed(2)) + " Bilder pro Sekunde von der gewünschten Bildrate ab.", windowWidth * 0.5,
    img_video_height * 0.5);
  text("Stelle eine andere Bildrate ein, lade die Seite neu,", windowWidth * 0.5,
    img_video_height * 0.5 + textsize * 0.3 * margin + 2 * margin);
  text("oder verwende einen anderen Browser.", windowWidth * 0.5,
    img_video_height * 0.5 + 2 * (textsize * 0.3 * margin + 2 * margin));
  pop();
}




function renderMagnifier() {
  if (show_magnifier) {
    let magn_x_pos = mouseX - magn_width * 0.5;
    let magn_y_pos;
    if (mouseX > windowWidth - magn_width * 0.5)
      magn_x_pos = mouseX - magn_width - margin;
    else if (mouseX < magn_width * 0.5)
      magn_x_pos = mouseX + margin;

    push();
    strokeWeight(1);
    fill(0);
    stroke(0);
    if (mouseY > magn_height * 1.2 + 4 * margin) {
      magn_y_pos = mouseY - magn_height * 1.2 - 4 * margin;
      triangle(mouseX, mouseY, magn_x_pos + magn_width * 0.5 - margin, magn_y_pos + magn_height, magn_x_pos + magn_width * 0.5 + margin, magn_y_pos + magn_height);
    } else {
      magn_y_pos = mouseY + magn_height;
      triangle(mouseX, mouseY, magn_x_pos + magn_width * 0.5 - margin, magn_y_pos, magn_x_pos + magn_width * 0.5 + margin, magn_y_pos);
    }
    pop();

    push();
    fill(255);
    stroke(0);
    strokeWeight(3);
    strokeCap(ROUND);
    rect(magn_x_pos, magn_y_pos,
      magn_width, magn_height);
    pop();

    image(img_big,
      magn_x_pos, magn_y_pos,
      magn_width, magn_height,
      ((mouseX - magn_width * magnification * 0.5)) + magn_width,
      ((mouseY - magn_height * magnification * 0.5)) + magn_height,
      magn_width * magnification,
      magn_height * magnification);

    push();
    strokeWeight(1);
    fill(0, 160);
    stroke(0, 160);
    line(magn_x_pos + magn_width * 0.5, magn_y_pos,
      magn_x_pos + magn_width * 0.5, magn_y_pos + magn_height);
    line(magn_x_pos, magn_y_pos + magn_height * 0.5,
      magn_x_pos + magn_width, magn_y_pos + magn_height * 0.5);
    stroke(255, 160);
    circle(magn_x_pos + magn_width * 0.5, magn_y_pos + magn_height * 0.5, margin * 0.8);
    noFill();
    stroke(0, 160);
    circle(magn_x_pos + magn_width * 0.5, magn_y_pos + magn_height * 0.5, magn_height * 0.8);
    pop();
  }
}




function renderFPS() {
  push();
  fill(0);
  stroke(255);
  strokeWeight(2);
  textAlign(RIGHT);
  textSize(textsize);
  textFont('Arial');
  textStyle(NORMAL);
  text(str(fps) + " fps", (windowWidth + img_video_width) * 0.5 - margin, img_video_height - margin - textsize);
  if (fps == 1)
    text("1 Sekunde pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else if (fps == 2)
    text("0,5 Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else if (fps == 4)
    text("0,25 Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else if (fps == 5)
    text("0,2 Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else if (fps == 8)
    text("0,125 Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else if (fps == 10)
    text("0,1 Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else if (fps <= 10)
    text("0," + str(round((1 / fps) * 10000)) + " Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  else
    text("0,0" + str(round((1 / fps) * 10000)) + " Sekunden pro Bild", (windowWidth + img_video_width) * 0.5 - margin, img_video_height);
  pop();
}



function pressed() {
  if (render == 0 && !render_reference) {
    if (touches.length != 0)
      always_show_magnifier = true;

    img_canvas = createImage(width, height);

    img_big = createImage(width + 2 * magn_width, height + 2 * magn_height);

    //canvas to img_canvas
    loadPixels();
    img_canvas.loadPixels();
    let d = pixelDensity();
    for (let x = 0; x < width; x++) {
      for (let y = 0; y < height; y++) {
        for (let i = 0; i < d; i++) {
          for (let j = 0; j < d; j++) {
            let index = 4 * ((y * d + j) * width * d + (x * d + i));
            let ind = 4 * ((y + j) * width + (x + i));
            img_canvas.pixels[ind] = pixels[index];
            img_canvas.pixels[ind + 1] = pixels[index + 1];
            img_canvas.pixels[ind + 2] = pixels[index + 2];
            img_canvas.pixels[ind + 3] = pixels[index + 3];
          }
        }
      }
    }
    img_canvas.updatePixels();

    img_big.copy(img_canvas, 0, 0, width, height, magn_width, magn_height, width, height);


    if (mouseY < img_video_height + 2 * margin) {
      if (always_show_magnifier)
        show_magnifier = true;

      if (display_mode == 0) {
        if (vectors[6].isNearStart(mouseX, mouseY))
          vectors[6].changeStartHighlighted(true);
        else if (vectors[7].isNearStart(mouseX, mouseY))
          vectors[7].changeStartHighlighted(true);

        else if (vectors[6].isNearEnd(mouseX, mouseY))
          vectors[6].changeEndHighlighted(true);
        else if (vectors[7].isNearEnd(mouseX, mouseY))
          vectors[7].changeEndHighlighted(true);

        else if (vectors[6].isNearMiddle(mouseX, mouseY))
          vectors[6].changeMiddleHighlighted(true);
        else if (vectors[7].isNearMiddle(mouseX, mouseY))
          vectors[7].changeMiddleHighlighted(true);
    
      } else if (display_mode > 0 && display_mode < 4) {
        if (vectors[0].isNearStart(mouseX, mouseY))
          vectors[0].changeStartHighlighted(true);
        else if (vectors[1].isNearStart(mouseX, mouseY))
          vectors[1].changeStartHighlighted(true);
        else if (vectors[2].isNearStart(mouseX, mouseY))
          vectors[2].changeStartHighlighted(true);
        else if (vectors[3].isNearStart(mouseX, mouseY))
          vectors[3].changeStartHighlighted(true);
        else if (vectors[4].isNearStart(mouseX, mouseY))
          vectors[4].changeStartHighlighted(true);
        else if (vectors[5].isNearStart(mouseX, mouseY))
          vectors[5].changeStartHighlighted(true);
					
				else if (display_mode == 3 && vectors[6].isNearStart(mouseX, mouseY))
          vectors[6].changeStartHighlighted(true);

        else if (vectors[0].isNearEnd(mouseX, mouseY))
          vectors[0].changeEndHighlighted(true);
        else if (vectors[1].isNearEnd(mouseX, mouseY))
          vectors[1].changeEndHighlighted(true);
        else if (vectors[2].isNearEnd(mouseX, mouseY))
          vectors[2].changeEndHighlighted(true);
        else if (vectors[3].isNearEnd(mouseX, mouseY))
          vectors[3].changeEndHighlighted(true);
        else if (vectors[4].isNearEnd(mouseX, mouseY))
          vectors[4].changeEndHighlighted(true);
        else if (vectors[5].isNearEnd(mouseX, mouseY))
          vectors[5].changeEndHighlighted(true);
				else if (display_mode == 3 && vectors[6].isNearEnd(mouseX, mouseY))
          vectors[6].changeEndHighlighted(true);

        else if (vectors[0].isNearMiddle(mouseX, mouseY))
          vectors[0].changeMiddleHighlighted(true);
        else if (vectors[1].isNearMiddle(mouseX, mouseY))
          vectors[1].changeMiddleHighlighted(true);
        else if (vectors[2].isNearMiddle(mouseX, mouseY))
          vectors[2].changeMiddleHighlighted(true);
        else if (vectors[3].isNearMiddle(mouseX, mouseY))
          vectors[3].changeMiddleHighlighted(true);
        else if (vectors[4].isNearMiddle(mouseX, mouseY))
          vectors[4].changeMiddleHighlighted(true);
        else if (vectors[5].isNearMiddle(mouseX, mouseY))
          vectors[5].changeMiddleHighlighted(true);
				else if (display_mode == 3 && vectors[6].isNearMiddle(mouseX, mouseY))
          vectors[6].changeMiddleHighlighted(true);
      }
    }
    slider.checkPressed();
    slider.updateValue();
  }
}


function mousePressed() {
	pressed();
}


function touchStarted() {
	pressed();
}


function dragged() {
  if (mouseY < img_video_height + 2 * margin) {
    if (always_show_magnifier)
      show_magnifier = true;

    let loop_start;
    let loop_end;
    if (display_mode == 0) {
      loop_start = 6;
      loop_end = 7;
    } else if (display_mode > 0 && display_mode < 4) {
      loop_start = 0;
      loop_end = 6;
    }
    for (let i = loop_start; i < loop_end + 1; i++) {
      if (vectors[i].isStartHighlighted())
        vectors[i].setStart(mouseX, mouseY);
      else if (vectors[i].isEndHighlighted())
        vectors[i].setEnd(mouseX, mouseY);
      else if (vectors[i].isMiddleHighlighted())
        vectors[i].move(mouseX, mouseY);
    }
    if (vectors[6].isStartHighlighted() || vectors[6].isEndHighlighted() || vectors[6].isMiddleHighlighted())
      scale_length = vectors[6].getLength();
  }
	
   slider.updateValue();
}


function mouseDragged() {
  dragged();
}


function touchMoved() {
  dragged();

  //diable scrolling
  if (mouseX > (windowWidth - img_video_width) * 0.5 - margin &&
    mouseX < (windowWidth + img_video_width) * 0.5 + margin) {
    return false;
  }
}

function released() {
	if (always_show_magnifier)
    show_magnifier = false;

  for (let i = 0; i < 8; i++) {
    vectors[i].changeMiddleHighlighted(false);
    vectors[i].changeStartHighlighted(false);
    vectors[i].changeEndHighlighted(false);
  }

  slider.pressed = false;
}


function mouseReleased() {
  released();
}


function touchEnded() {
	released();
}



function keyPressed() {
  if (keyCode == TAB) {
    let fs = fullscreen();
    fullscreen(!fs);
  }

  if (keyCode == 32) {
    always_show_magnifier ^= true
  }
}



function deviceShaken() {
  fullscreen(true);
}



function saveImage() {
  resizeCanvas(windowWidth, img_video_height + 2 * margin);
  let file_name = year() + "_" + month() + "_" + day() + "_" + hour() + "_" + minute() + "_" + second() + "_Stroboskop_Bild";
  saveCanvas(file_name, 'png');
  resizeCanvas(windowWidth, windowHeight);
}



function buttonDisplayMode() {
  display_mode++;
  if (display_mode > 4)
    display_mode = 0;

  if (display_mode == 0)
    vectors[6].input.show();

  if (display_mode == 1) {
    for (let i = 0; i < 6; i++) {
      vectors[i].changeShowText(false);
      vectors[i].changeStyle(0);
    }
    vectors[6].input.hide();
  }

  if (display_mode == 2) {
    for (let i = 0; i < 6; i++) {
      vectors[i].changeShowText(true);
			vectors[i].changeStyle(0);
		}
    vectors[6].input.hide();
  }
	
	if (display_mode == 3) {
    for (let i = 0; i < 6; i++) {
      vectors[i].changeShowText(true);
			vectors[i].changeStyle(3);
		}
    vectors[6].input.show();
  }
	
	if (display_mode == 4) {
		vectors[6].input.hide();
	}
}


function buttonInterfacePosition() {
  interface_up ^= 1;
  resizeLayout();
}


function defineVectors() {
  vectors[0] = new Vector(windowWidth * 0.2, img_video_height * 0.5,
    windowWidth * 0.2, img_video_height * 0.1,
    2, color(220, 0, 0), img_video_width * margin_relation, textsize * 0.9, 0);

  vectors[1] = new Vector(windowWidth * 0.2, img_video_height * 0.1,
    windowWidth * 0.4, img_video_height * 0.1,
    2, color(0, 220, 0), img_video_width * margin_relation, textsize * 0.9, 0);

  vectors[2] = new Vector(windowWidth * 0.2, img_video_height * 0.5,
    windowWidth * 0.4, img_video_height * 0.1,
    2, color(0, 0, 220), img_video_width * margin_relation, textsize * 0.9, 0);

  vectors[3] = new Vector(windowWidth * 0.6, img_video_height * 0.5,
    windowWidth * 0.6, img_video_height * 0.1,
    2, color(170, 0, 170), img_video_width * margin_relation, textsize * 0.9, 0);

  vectors[4] = new Vector(windowWidth * 0.6, img_video_height * 0.1,
    windowWidth * 0.8, img_video_height * 0.1,
    2, color(170, 170, 0), img_video_width * margin_relation, textsize * 0.9, 0);

  vectors[5] = new Vector(windowWidth * 0.6, img_video_height * 0.5,
    windowWidth * 0.8, img_video_height * 0.1,
    2, color(0, 170, 170), img_video_width * margin_relation, textsize * 0.9, 0);

  vectors[6] = new Vector(windowWidth * 0.5 - img_video_width * 0.4, img_video_height * 0.34,
    windowWidth * 0.5 + img_video_width * 0.4, img_video_height * 0.34,
    2, color(0), img_video_width * margin_relation, textsize, 2);
  vectors[6].changeShowText(true);
  scale_length = vectors[6].getLength();
  scale = scale_value / scale_length;
	
	vectors[7] = new Vector(windowWidth * 0.5 - img_video_width * 0.4, img_video_height * 0.09,
    windowWidth * 0.5 + img_video_width * 0.4, img_video_height * 0.09,
    2, color(0, 0, 220), img_video_width * margin_relation, textsize, 1);
  vectors[7].changeShowText(true);
}



function resizeLayout() {
  margin = img_video_width * margin_relation;
  button_width = (img_video_width - 7 * margin) / 7.5;
  button_height = textsize;
  magn_width = img_video_width * 0.2;
  magn_height = img_video_height * 0.2;

  for (let i = 0; i < 8; i++)
    vectors[i].changeThreshold(margin * 5);

  let button_position_x = (windowWidth - img_video_width - 2 * margin) * 0.5
  let button_position_y = img_video_height + 1.75 * margin - interface_up * button_height * 4.5;

  input.position(margin + button_position_x, button_position_y);
  input.size(2 * button_width + margin);

  button_render.position(4 * button_width + 5 * margin + button_position_x,
    button_position_y);
  button_render.size(button_width);
  button_save.position(5 * button_width + 6 * margin + button_position_x,
    button_position_y);
  button_save.size(button_width);
  button_display_mode.position(6 * button_width + 7 * margin + button_position_x,
    button_position_y);
  button_display_mode.size(button_width);
  button_up_down.position(7 * button_width + 8 * margin + button_position_x,
    button_position_y);
  button_up_down.size(button_width * 0.5);

  slider = {
    pressed: false,
    x: 2 * button_width + 3 * margin + button_position_x,
    y: button_position_y + margin * 0.5,
    x_offset: button_height * 0.95 * 0.5,
    width: button_width * 2,
    height: button_height * 0.95,
    intervall: (button_width * 2 - button_height * 0.95) / (fps_max - 1),
    value: fps,

    render: function () {
      push();
      rectMode(CORNER);
      fill(245);
      stroke(0);
      this.pressed ? strokeWeight(2) : strokeWeight(1);
      rect(this.x, this.y + this.height * 0.35, this.width, this.height * 0.3, this.height);
      fill(225);
      ellipse(this.x + (this.value - 1) * this.intervall + this.x_offset, this.y + this.height * 0.5, this.height);
      pop();
    },

    updateValue: function () {
      if (this.pressed)
        this.value = constrain(round(map(mouseX, this.x + this.x_offset, this.x + this.width - this.x_offset, 1, fps_max)), 1, fps_max);
				if(this.value > 1 && this.value % 2 != 0)
					this.value -= 1;
    },

    checkPressed: function () {
      if (mouseX > this.x && mouseX < this.x + this.width && mouseY > this.y && mouseY < this.y + this.height)
        this.pressed = true;
      else
        this.pressed = false;
    }
  }
}



function windowResized() {
  resizeCanvas(windowWidth, windowHeight);

  if (windowHeight >= windowWidth * (1 + 2 * (margin_relation)) * 0.57 + textsize) {
    img_video_width = windowWidth * (1 - 2 * (margin_relation));
    img_video_height = (windowWidth * (1 - 2 * (margin_relation))) * 0.57;
  } else {
    img_video_width = (windowHeight - textsize - 5 * windowHeight * margin_relation) * 1.78;
    img_video_height = windowHeight - textsize - 5 * windowHeight * margin_relation;
  }

  resizeLayout();
}