/*
  StrobeVideo

  Erstellt ein Stroboskopbild aus einer Videodatei und lässt stellt Werkzeuge zur Verfügung, um dieses Bild im Sinne einer Videoanalyse auszuwerten.

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


p5.disableFriendlyErrors = true; // disables FES


function preload() {
  video = createVideo('videos/example.mp4');
}



function setup() {
  createCanvas(windowWidth, windowHeight);

  if (windowHeight >= windowWidth * (1 + 2 * (margin_relation)) * 0.57 + textsize) {
    img_video_width = windowWidth * (1 - 2 * (margin_relation));
    img_video_height = (windowWidth * (1 - 2 * (margin_relation))) * 0.57;
  } else {
    img_video_width = (windowHeight - textsize - 5 * windowHeight * margin_relation) * 1.78;
    img_video_height = windowHeight - textsize - 5 * windowHeight * margin_relation;
  }

  fps = 4;

  input = createFileInput(handleFile);

  button_render = createButton('Rendern');
  button_render.mousePressed(renderImage);

  button_save = createButton('Speichern');
  button_save.mousePressed(saveImage);

  button_display_mode = createButton('Ansicht');
  button_display_mode.mousePressed(buttonDisplayMode);

  button_up_down = createButton('↑↓');
  button_up_down.mousePressed(buttonInterfacePosition);

  interface_up = 0;

  video.hide();

  img = createImage(1280, 720);
  img_reference = createImage(1280, 720);

  defineVectors();

  resizeLayout();

  always_show_magnifier = false;
  show_magnifier = false;
  display_mode = 0;

  render_reference = false;
  render = 0;
  new_video = true;

  frameRate(fps_default);
}




function draw() {
  if (render == 0 && !render_reference) {
    background(255);
    fps = slider.value;
  }

  if (video) {
    //Referenz-Bild wird erzeugt
    if (render_reference) {
      vectors[6].input.hide();
      renderReference();
    }

    //Während das Video durchläuft werden die Frames analysiert
    if (render > 0 && !render_reference) {
      vectors[6].input.hide();
      analyseFrame();
      video.onended(pauseRender);
      show_magnifier = false;
      framerates.push(frameRate());
      image(img, (windowWidth - img_video_width - 2 * margin) * 0.5 + margin, margin, img_video_width, img_video_height);
    } else {
      frameRate(fps_default);
    }

    //Startbildschirm mit Erklärungen oder fertig analysiertes Bild
    if (render == 0 && !render_reference) {
      if (!framerate_error) {
        //Startbildschirm
        renderInstructions();

        //fertig analysiertes Bild
        image(img, (windowWidth - img_video_width - 2 * margin) * 0.5 + margin, margin, img_video_width, img_video_height);

      //Fehlerausgabe
      } else {
        image(img, (windowWidth - img_video_width - 2 * margin) * 0.5 + margin, margin, img_video_width, img_video_height);
        renderError();
      }
    }
  }

  //FPS, Vektoren und die Lupe werden angezeigt
  if (render == 0 && !render_reference) {
    renderFPS();

    if (display_mode == 0) {
      vectors[6].input.show();
      scale = scale_value / scale_length;
      for (let i = 6; i < 8; i++)
        vectors[i].renderVector();
    } else if (display_mode > 0 && display_mode < 4) {
			if (display_mode == 3) {
      	vectors[6].input.show();
				vectors[6].renderVector();
				scale = scale_value / scale_length
			} else
			vectors[6].input.hide();
			
      for (let i = 5; i >= 0; i--)
        vectors[i].renderVector();
    }
    slider.render();
    renderMagnifier();
  }
}