class Vector {
  constructor(x_start, y_start, x_end, y_end, line_width, c, size, textsize, style) {
    if (c == -1) {
      let tempcolor = [random(0, 255), random(0, 255), random(0, 255)]
      while (max(tempcolor) < 150)
        tempcolor = [random(0, 255), random(0, 255), random(0, 255)]
      this.c = color(tempcolor);
    } else {
      this.c = c;
    }

    this.x_start = x_start;
    this.y_start = y_start;
    this.x_end = x_end;
    this.y_end = y_end;
    if (line_width < 0)
      this.line_width = 2;
    else
      this.line_width = line_width;
    this.size = size;
    this.textsize = textsize;
    this.style = style;
    this.length = dist(x_start, y_start, x_end, y_end);

    this.startH = false;
    this.endH = false;
    this.middleH = false;
    this.threshold = 10;
    this.show_text = false;

    this.prevmousex = mouseX;
    this.prevmousey = mouseY;

    if (style == 2) {
      this.input = createInput('0.5');
      this.input.input(this.myInputEvent);
      this.input.size(this.textsize * 3, this.textsize * 0.9);
      this.input.style('text-align: center');
      this.input.style('font-size: ' + str(this.textsize) + 'px');
      this.input.style('color: ' + str(this.c));
      scale_value = this.input.value();
    }
  }


  myInputEvent() {
    scale_value = this.value();
  }

  distSqV(x1, y1, x2, y2) {
    return ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
  }

  changeThreshold(new_threshold) {
    this.threshold = new_threshold;
  }

  changeShowText(bool) {
    this.show_text = bool;
  }
	
	changeStyle(val) {
    this.style = val;
  }

  changeStartHighlighted(bool) {
    this.startH = bool;
  }

  changeEndHighlighted(bool) {
    this.endH = bool;
  }

  changeMiddleHighlighted(bool) {
    this.middleH = bool;
  }

  isStartHighlighted() {
    return this.startH;
  }

  isEndHighlighted() {
    return this.endH;
  }

  isMiddleHighlighted() {
    return this.middleH;
  }

  move(_x, _y) {
    let deltax = _x - this.prevmousex;
    let deltay = _y - this.prevmousey;
    this.x_start += deltax;
    this.y_start += deltay;
    this.x_end += deltax;
    this.y_end += deltay;
		this.prevmousex = _x;
    this.prevmousey = _y;
  }

  setStart(_x, _y) {
    this.x_start = _x;
    this.y_start = _y;
    this.length = this.getLength();
  }

  setEnd(_x, _y) {
    this.x_end = _x;
    this.y_end = _y;
    this.length = this.getLength();
  }

  isNearStart(_x, _y) {
    let thres = this.threshold;
    if (this.threshold * 4 > this.length)
      thres = this.length * 0.25;

    if (this.distSqV(_x, _y, this.x_start, this.y_start) < thres * thres + 2)
      return true;
    else
      return false;
  }

  isNearEnd(_x, _y) {
    let thres = this.threshold;
    if (this.threshold * 4 > this.length)
      thres = this.length * 0.25;

    if (this.distSqV(_x, _y, this.x_end, this.y_end) < thres * thres)
      return true;
    else
      return false;
  }

  isNearMiddle(_x, _y) {
    let thres = this.threshold;

    let x_middle = this.x_start + (this.x_end - this.x_start) * 0.5;
    let y_middle = this.y_start + (this.y_end - this.y_start) * 0.5;

    let n_x = (this.y_start - this.y_end);
    let n_y = (this.x_start - this.x_end);
    let mag_n = mag(n_y, n_x)
    n_x = n_x / mag_n;
    n_y = n_y / mag_n;    

    if ((this.x_start * n_x - this.y_start * n_y) < 0) {
      n_x = (-1) * n_x;
      n_y = (-1) * n_y;
    }

    let d = n_x * this.x_start - n_y * this.y_start;
    
    if (this.threshold * 4 > this.length)
      thres = this.length * 0.25;

    if (abs((n_x * _x - n_y * _y) - d) < thres &&
      this.distSqV(_x, _y, x_middle, y_middle) < this.length * this.length * 0.25) {
      this.prevmousex = _x;
      this.prevmousey = _y;
      return true;
    } else {
      return false;
    }
  }

  getLength() {
    return dist(this.x_start, this.y_start, this.x_end, this.y_end);
  }

  renderVector() {
    push();
    stroke(this.c);
    fill(this.c);

    angleMode(DEGREES);
    let angle = atan2(this.y_end - this.y_start, this.x_end - this.x_start);

    push();
    if (this.middleH)
      strokeWeight(this.line_width * 2);
    else
      strokeWeight(this.line_width);
    line(this.x_start, this.y_start, this.x_end, this.y_end);
    pop();

    push()
    if (this.startH)
      strokeWeight(this.line_width * 2);
    else
      strokeWeight(this.line_width);
    translate(this.x_start, this.y_start);
    rotate(angle);
    line(0, 0 - this.size, 0, 0 + this.size);
    pop();
    if (this.style == 1 || this.style == 2) {
      if (this.endH)
        strokeWeight(this.line_width * 2);
      else
        strokeWeight(this.line_width);
      push();
      translate(this.x_end, this.y_end);
      rotate(angle);
      line(0, 0 - this.size, 0, 0 + this.size);
      pop();
    }

    push();
    if (this.endH)
      strokeWeight(this.line_width * 2);
    else
      strokeWeight(this.line_width);
    translate(this.x_end, this.y_end);
    rotate(angle);
    line(0, 0, 0 - this.size, 0 + this.size);
    line(0, 0, 0 - this.size, 0 - this.size);
    pop();
    if (this.style == 1 || this.style == 2) {
      push();
      if (this.startH)
        strokeWeight(this.line_width * 2);
      else
        strokeWeight(this.line_width);
      translate(this.x_start, this.y_start);
      rotate(angle);
      line(0, 0, 0 + this.size, 0 + this.size);
      line(0, 0, 0 + this.size, 0 - this.size);
      pop();
    }

    if (this.show_text) {
      push();
      strokeWeight(this.line_width);
      stroke(255);
      textSize(this.textsize);
      textFont('Arial');
      textStyle(NORMAL);
      textAlign(CENTER);
      if (this.x_start > this.x_end)
        angle += 180;
      translate(this.x_start + (this.x_end - this.x_start) * 0.5, this.y_start + (this.y_end - this.y_start) * 0.5);
      rotate(angle);
      if (this.style == 0)
        text(str(round(this.length)) + ' Pixel', 0, this.textsize * 1.2);
      else if (this.style == 1 || this.style == 3)
        text(str((this.length * scale).toFixed(3)) + ' m', 0, this.textsize * 1.2);
      pop();
      if (this.style == 2) {
        push();
        strokeWeight(this.line_width);
        stroke(255);
        textSize(this.textsize);
        textFont('Arial');
        textStyle(NORMAL);
        textAlign(RIGHT, TOP);
        fill(this.c);
        let temp_x = this.x_start + (this.x_end - this.x_start) * 0.5 - this.textsize * 2.5;
        let temp_y = this.y_start + (this.y_end - this.y_start) * 0.5 - this.textsize * 0.65;
        text(str(scale_value) + '    m', temp_x + this.textsize * 5.2, temp_y);
        this.input.position(temp_x, temp_y);
        pop();
      }
    }
    pop();
  }
}